/* GNU djinn - vector manipulation routines
   Copyright (C) 1998 Free Software Foundation, Inc.

   This file is part of GNU djinn

   GNU djinn is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GNU djinn is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU djinn; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
   
   The author may be contacted at:
 
   frenzy@ix.netcom.com
*/


#include <stdio.h>
#include <string.h>

#include "vector.h"

vec_t *vec_start(unsigned int resize_rate)
  {
  vec_t *v = (vec_t *)malloc(sizeof(vec_t));

  v->len = v->c = 0;
  v->blk = 1;
  v->blksz = resize_rate;
  v->vs = v->v = (char **)malloc(v->blksz * sizeof(char *));

  return(v);
  }

void vec_store(vec_t *v, const char *s)
  {
  *(v->v) = strdup(s);
  (v->len)++;

  if(++(v->c) == v->blksz)
    {
    v->c = 0;
    v->vs = (char **)realloc(v->vs, v->blksz * ++(v->blk) * sizeof(char *));
    v->v = v->vs + ((v->blk - 1) * v->blksz);
    }
  else (v->v)++;
  }

char **vec_end(vec_t *v, unsigned int *len)
  {
  char **r;
  
  if(len) *len = v->len;
  if(!v->len)
    {
    free(v->vs);
    free(v);
    return(NULL);
    }

  r = (char **)realloc(v->vs, ++(v->len) * sizeof(char *));
  *(r + v->len - 1) = NULL;
  free(v);

  return(r);
  }

void vec_print(FILE *fp, vec_t *v)
  {
  char **p;

  for(p = v->vs; *p; p++)
    {
    fprintf(fp, *p);
    fputc(' ', fp);
    }
  putchar('\n');
  }

void vec_abort(vec_t *v)
  {
     /* free the strings too */
  free(v->vs);
  free(v);
  }

